(
    CallerResult(
        self=CallerInfo(
            name='single_level',
            file='/Users/luckydonald/Documents/Programmieren/Python/luckydonald-utils.git/luckydonaldUtils/functions.py',
            line=82,
            code=None
        ),
        caller=CallerInfo(
            name='run',
            file='/Users/luckydonald/.pyenv/versions/3.7.2/lib/python3.7/unittest/case.py',
            line=615,
            code='                    testMethod()\n')),
    CallerResult(
        self=CallerInfo(
            name='single_level',
            file='/Users/luckydonald/Documents/Programmieren/Python/luckydonald-utils.git/tests/functions.py',
            line=7,
            code=None
        ), caller=CallerInfo(
            name='test_one_level',
            file='/Users/luckydonald/Documents/Programmieren/Python/luckydonald-utils.git/tests/functions.py',
            line=41,
            code='        result, result2 = single_level()\n'
        )
    )
)
